package com.diagbot.config.redislimiter;

import org.springframework.cloud.gateway.filter.ratelimit.KeyResolver;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.util.Iterator;
import java.util.LinkedHashSet;

/**
 * @Description:
 * @author: gaodm
 * @time: 2019/12/26 15:31
 */
public class SysKeyResolver implements KeyResolver {
    private static final String GATE_WAY_PREFIX = "/api";

    @Override
    public Mono<String> resolve(ServerWebExchange exchange) {
        LinkedHashSet requiredAttribute
                = exchange.getRequiredAttribute(ServerWebExchangeUtils.GATEWAY_ORIGINAL_REQUEST_URL_ATTR);
        String fullUrl = "";
        if (requiredAttribute != null) {
            Iterator<URI> iterator = requiredAttribute.iterator();
            while (iterator.hasNext()) {
                URI next = iterator.next();
                if (next.getPath().startsWith(GATE_WAY_PREFIX)) {
                    fullUrl = next.getPath();
                }
            }
        }

        String myKeyResolver = exchange.getRequest().getRemoteAddress().getAddress().getHostAddress()
                + "_" + fullUrl;
        ServerHttpRequest.Builder builder = exchange.getRequest().mutate();
        builder.header("myKeyResolver", myKeyResolver);
        exchange.mutate().request(builder.build()).build();

        return Mono.just(exchange.getRequest().getHeaders().getFirst("myKeyResolver"));
    }
}